@@ -37,9 +37,11 @@ namespace net6
 /** Client in a Client/Server based TCP network.
  */
 template<typename selector_type>
-class basic_client : virtual public basic_local<selector_type>
+class basic_client: virtual public basic_local<selector_type>
 {
 public:
+	typedef connection<selector_type> connection_type;
+
 	typedef sigc::signal<void, const user&, const packet&>
 		signal_join_type;
 	typedef sigc::signal<void, const user&, const packet&>
@@ -48,6 +50,8 @@ public:
 		signal_data_type;
 	typedef sigc::signal<void>
 		signal_close_type;
+	typedef sigc::signal<void>
+		signal_encrypted_type;
 	typedef sigc::signal<void, login::error>
 		signal_login_failed_type;
 	typedef sigc::signal<void, packet&>
@@ -80,12 +84,21 @@ public:
 	 */
 	bool is_connected() const;
 
+	/** Requests encryption from the peer. An encrypted_event is
+	 * emitted as soon as the connection is guranteed to be encrypted.
+	 */
+	void request_encryption();
+
 	/** Send a login request with the specified user name. On success,
 	 * a join_event with user==self is emitted, otherwise a
 	 * login_failed_event.
 	 */
 	void login(const std::string& username);
 
+	/** Returns whether the local user is logged in.
+	 */
+	bool is_logged_in() const;
+
 	/** Send a packet to the network server.
 	 */
 	virtual void send(const packet& pack);
@@ -102,8 +115,13 @@ public:
 
 	/** Returns the underlaying net6::connection object.
 	 */
-	const connection& get_connection() const;
-	
+	const connection_type& get_connection() const;
+
+	/** @brief Sets whether to send keepalives to the server when
+	 * the connection is inactive.
+	 */
+	void set_enable_keepalives(bool enable);
+
 	/** Signal which is emitted every time a client joins the network.
 	 */
 	signal_join_type join_event() const;
@@ -122,6 +140,11 @@ public:
 	 */
 	signal_close_type close_event() const;
 
+	/** Signal that will be emitted when the encryption to the server
+	 * is guaranteed to be secure.
+	 */
+	signal_encrypted_type encrypted_event() const;
+
 	/** Signal which is emitted, if a login request failed, for example
 	 * if the wished user name was already in use by another client.
 	 */
@@ -138,35 +161,39 @@ public:
 protected:
 	/** Signal handler that is called each time a packet arrives.
 	 */
-	virtual void on_recv_event(const packet& pack);
-
-	/** Signal handler that is called when all available data has been sent.
-	 */
-	virtual void on_send_event();
+	void on_recv_event(const packet& pack);
 
 	/** Signal handler that is called when the remote site closed the
 	 * connection.
 	 */
-	virtual void on_close_event();
+	void on_close_event();
+
+	/** Signal handler that is called when a secure connection is
+	 * guaranteed to be secure.
+	 */
+	void on_encrypted_event();
 
 	virtual void on_join(const user& user, const packet& pack);
 	virtual void on_part(const user& user, const packet& pack);
 	virtual void on_data(const packet& pack);
 	virtual void on_close();
+	virtual void on_encrypted();
 	virtual void on_login_failed(login::error error);
 	virtual void on_login_extend(packet& pack);
 
-	virtual void net_login_failed(const packet& pack);
-	virtual void net_client_join(const packet& pack);
-	virtual void net_client_part(const packet& pack);
+	void net_login_failed(const packet& pack);
+	void net_client_join(const packet& pack);
+	void net_client_part(const packet& pack);
+	void net_encryption_info(const packet& pack);
 
-	std::auto_ptr<connection> conn;
+	std::auto_ptr<connection_type> conn;
 	user* self;
 
 	signal_join_type signal_join;
 	signal_part_type signal_part;
 	signal_data_type signal_data;
 	signal_close_type signal_close;
+	signal_encrypted_type signal_encrypted;
 	signal_login_failed_type signal_login_failed;
 	signal_login_extend_type signal_login_extend;
 
@@ -178,14 +205,14 @@ private:
 typedef basic_client<selector> client;
 
 template<typename selector_type>
-basic_client<selector_type>::basic_client()
- : basic_local<selector_type>(), self(NULL)
+basic_client<selector_type>::basic_client():
+	basic_local<selector_type>(), self(NULL)
 {
 }
 
 template<typename selector_type>
-basic_client<selector_type>::basic_client(const net6::address& addr)
- : basic_local<selector_type>(), self(NULL)
+basic_client<selector_type>::basic_client(const net6::address& addr):
+	basic_local<selector_type>(), self(NULL)
 {
 	connect_impl(addr);
 }
@@ -215,6 +242,16 @@ bool basic_client<selector_type>::is_connected() const
 	return conn.get() != NULL;
 }
 
+template<typename selector_type>
+void basic_client<selector_type>::request_encryption()
+{
+	// It is sufficient to request encryption by sending
+	// the corresponding packet to the server. The handler
+	// code in the connection will take care of establishing
+	// the additional security layer.
+	conn->request_encryption(true);
+}
+
 template<typename selector_type>
 void basic_client<selector_type>::login(const std::string& username)
 {
@@ -224,15 +261,15 @@ void basic_client<selector_type>::login(const std::string& username)
 	send(login_pack);
 }
 
+template<typename selector_type>
+bool basic_client<selector_type>::is_logged_in() const
+{
+	return conn.get() != NULL && self != NULL;
+}
+
 template<typename selector_type>
 void basic_client<selector_type>::send(const packet& pack)
 {
-	selector_type& selector = basic_object<selector_type>::get_selector();
-
-	// Add OUTGOING flag it to the selector if it isn't already set
-	if(!selector.check(conn->get_socket(), socket::OUTGOING) )
-		selector.add(conn->get_socket(), socket::OUTGOING);
-
 	// Add packet to send queue
 	conn->send(pack);
 }
@@ -258,7 +295,8 @@ const user& basic_client<selector_type>::get_self() const
 }
 
 template<typename selector_type>
-const connection& basic_client<selector_type>::get_connection() const
+const typename basic_client<selector_type>::connection_type&
+basic_client<selector_type>::get_connection() const
 {
 	if(!is_connected() )
 		throw not_connected_error("net6::basic_client::get_connection");
@@ -266,6 +304,19 @@ const connection& basic_client<selector_type>::get_connection() const
 	return *conn;
 }
 
+template<typename selector_type>
+void basic_client<selector_type>::set_enable_keepalives(bool enable)
+{
+	if(!is_connected() )
+	{
+		throw not_connected_error(
+			"net6::basic_client::set_enable_keepalives"
+		);
+	}
+
+	conn->set_enable_keepalives(enable);
+}
+
 template<typename selector_type>
 typename basic_client<selector_type>::signal_join_type
 basic_client<selector_type>::join_event() const
@@ -294,6 +345,13 @@ basic_client<selector_type>::close_event() const
 	return signal_close;
 }
 
+template<typename selector_type>
+typename basic_client<selector_type>::signal_encrypted_type
+basic_client<selector_type>::encrypted_event() const
+{
+	return signal_encrypted;
+}
+
 template<typename selector_type>
 typename basic_client<selector_type>::signal_login_failed_type
 basic_client<selector_type>::login_failed_event() const
@@ -317,20 +375,12 @@ void basic_client<selector_type>::on_recv_event(const packet& pack)
 		net_client_join(pack);
 	else if(pack.get_command() == "net6_client_part")
 		net_client_part(pack);
+	else if(pack.get_command() == "net6_encryption_info")
+		net_encryption_info(pack);
 	else
 		on_data(pack);
 }
 
-template<typename selector_type>
-void basic_client<selector_type>::on_send_event()
-{
-	selector_type& selector = basic_object<selector_type>::get_selector();
-
-	// Remove OUTGOING flag from selector as there is no more
-	// data to send
-	selector.remove(conn->get_socket(), socket::OUTGOING);
-}
-
 template<typename selector_type>
 void basic_client<selector_type>::on_close_event()
 {
@@ -340,6 +390,12 @@ void basic_client<selector_type>::on_close_event()
 	on_close();
 }
 
+template<typename selector_type>
+void basic_client<selector_type>::on_encrypted_event()
+{
+	on_encrypted();
+}
+
 template<typename selector_type>
 void basic_client<selector_type>::on_join(const user& user, const packet& pack)
 {
@@ -364,6 +420,12 @@ void basic_client<selector_type>::on_close()
 	signal_close.emit();
 }
 
+template<typename selector_type>
+void basic_client<selector_type>::on_encrypted()
+{
+	signal_encrypted.emit();
+}
+
 template<typename selector_type>
 void basic_client<selector_type>::on_login_failed(login::error error)
 {
@@ -391,9 +453,9 @@ template<typename selector_type>
 void basic_client<selector_type>::net_client_join(const packet& pack)
 {
 	// Received client_join packet
-	int id = pack.get_param(0).parameter::as<int>();
-	const std::string& name =
-		pack.get_param(1).parameter::as<std::string>();
+	unsigned int id = pack.get_param(0).parameter::as<int>();
+	std::string name = pack.get_param(1).parameter::as<std::string>();
+	bool is_encrypted = pack.get_param(2).parameter::as<bool>();
 
 	user* new_client = new user(id, NULL);
 	basic_object<selector_type>::user_add(new_client);
@@ -402,6 +464,11 @@ void basic_client<selector_type>::net_client_join(const packet& pack)
 	// The first client who joins is the local client
 	if(self == NULL) self = new_client;
 	on_join(*new_client, pack);
+
+	if(is_encrypted)
+	{
+		new_client->set_encrypted();
+	}
 }
 
 template<typename selector_type>
@@ -417,6 +484,18 @@ void basic_client<selector_type>::net_client_part(const packet& pack)
 	basic_object<selector_type>::user_remove(rem_user);
 }
 
+template<typename selector_type>
+void basic_client<selector_type>::net_encryption_info(const packet& pack)
+{
+	unsigned int id = pack.get_param(0).parameter::as<int>();
+	user* encr_user = basic_object<selector_type>::user_find(id);
+
+	if(encr_user == NULL)
+		throw bad_value("Got encryption_info for nonexistant user");
+
+	encr_user->set_encrypted();
+}
+
 template<typename selector_type>
 void basic_client<selector_type>::connect_impl(const address& addr)
 {
@@ -425,21 +504,21 @@ void basic_client<selector_type>::connect_impl(const address& addr)
 		throw connected_error("net6::basic_client::connect");
 
 	// Connect to remote host
-	conn.reset(new connection(addr) );
-
-	// Add socket to selector
-	basic_object<selector_type>::get_selector().add(
-		conn->get_socket(),
-		socket::INCOMING | socket::IOERROR
+	conn.reset(
+		new connection_type(
+			basic_object<selector_type>::get_selector()
+		)
 	);
 
 	// Install signal handlers
-	conn->send_event().connect(
-		sigc::mem_fun(*this, &basic_client::on_send_event) );
 	conn->recv_event().connect(
 		sigc::mem_fun(*this, &basic_client::on_recv_event) );
 	conn->close_event().connect(
 		sigc::mem_fun(*this, &basic_client::on_close_event) );
+	conn->encrypted_event().connect(
+		sigc::mem_fun(*this, &basic_client::on_encrypted_event) );
+
+	conn->connect(addr);
 }
 
 template<typename selector_type>
