@@ -47,11 +47,21 @@ public:
 	basic_host(unsigned int port, const std::string& username,
 	           bool ipv6 = true);
 
+	/** Sends a packet to all users.
+	 */
+	virtual void send(const packet& pack);
+
 	/** Sends a packet to a single user. The request is ignored if
 	 * <em>to</em> is the local user.
 	 */
 	virtual void send(const packet& pack, const user& to);
 
+	/** @brief Requests encryption to the given user.
+	 *
+	 * Throws an error if <em>to</em> is the local user.
+	 */
+	virtual void request_encryption(const user& to);
+
 	/** Returns the local user.
 	 */
 	virtual user& get_self();
@@ -89,12 +99,32 @@ basic_host<selector_type>::
 	basic_object<selector_type>::user_add(self);
 }
 
+template<typename selector_type>
+void basic_host<selector_type>::send(const packet& pack)
+{
+	basic_server<selector_type>::send(pack);
+}
+
 template<typename selector_type>
 void basic_host<selector_type>::send(const packet& pack, const user& to)
 {
 	if(&to != self) basic_server<selector_type>::send(pack, to);
 }
 
+template<typename selector_type>
+void basic_host<selector_type>::request_encryption(const user& to)
+{
+	if(&to == self)
+	{
+		throw std::logic_error(
+			"net6::basic_host::request_encryption:\n"
+			"Cannot request encryption to local client"
+		);
+	}
+
+	basic_server<selector_type>::request_encryption(to);
+}
+
 template<typename selector_type>
 user& basic_host<selector_type>::get_self()
 {
