@@ -37,12 +37,27 @@
 namespace net6
 {
 
+/** Packet handlers may throw this error if they received an invalid packet.
+ *
+ * The server will then close the connection to the client it comes from.
+ * Use this error in case where you cannot ensure synchonisation anymore. If
+ * the connection to the client is still valid, net6::bad_value is preferable.
+ */
+class bad_packet: public std::runtime_error
+{
+public:
+	bad_packet(const std::string& reason):
+		std::runtime_error(reason) {}
+};
+
 /** High-level TCP dedicated server object.
  */
 template<typename selector_type>
 class basic_server : virtual public basic_object<selector_type>
 {
 public:
+	typedef connection<selector_type> connection_type;
+
 	typedef default_accumulator<bool, true> auth_accumulator;
 
 	typedef sigc::signal<void, const user&>
@@ -66,7 +81,7 @@ public:
 
 	typedef sigc::signal<void, const user&, const packet&>
 		signal_data_type;
-	
+
 	/** Creates a new basic_server object.
 	 * @param ipv6 Whether to use IPv6.
 	 */
@@ -106,6 +121,10 @@ public:
 	 */
 	virtual void send(const packet& pack, const user& to);
 
+	/** @brief Requests secure communication with the given user.
+	 */
+	virtual void request_encryption(const user& to);
+
 	/** Returns the underlaying TCP server socket object. The function
 	 * throws not_connected_error if the server has not been opened.
 	 */
@@ -162,9 +181,10 @@ public:
 	/** Signal which may be used to append parameters to a client_join
 	 * packet which will be sent to existing users to announce the new join.
 	 * The first parameter is the new client's ID number, the second one
-	 * its user name, other parameters may be appended by you. The user
-	 * given to the signal handler is the use for which information has
-	 * to be appended, not the one to which they will be sent.
+	 * its user name, the third its encryption state. Other parameters may
+	 * be appended by you. The user given to the signal handler is the use
+	 * for which information has to be appended, not the one to which they
+	 * will be sent.
 	 */
 	signal_login_extend_type login_extend_event() const;
 
@@ -174,13 +194,13 @@ public:
 	signal_data_type data_event() const;
 	
 protected:
-	virtual void remove_client(const user* client);
+	void remove_client(const user* client);
 
-	virtual void on_accept_event(socket::condition io);
-	virtual void on_send_event(user& to);
-	virtual void on_recv_event(const packet& pack,
-	                           user& from);
-	virtual void on_close_event(user& user);
+	void on_accept_event(tcp_server_socket& sock, io_condition io);
+	void on_recv_event(const packet& pack,
+	                   user& from);
+	void on_close_event(user& user);
+	void on_encrypted_event(user& user);
 
 	virtual void on_connect(const user& user);
 	virtual void on_disconnect(const user& user);
@@ -199,9 +219,13 @@ protected:
 	virtual void net_client_login(user& from, const packet& pack);
 
 	std::auto_ptr<tcp_server_socket> serv_sock;
+	std::auto_ptr<tcp_server_socket> serv6_sock;
+
 	bool use_ipv6;
 	unsigned int id_counter;
 
+	dh_params params;
+
 	signal_connect_type signal_connect;
 	signal_disconnect_type signal_disconnect;
 	signal_join_type signal_join;
@@ -279,18 +303,16 @@ void basic_server<selector_type>::send(const packet& pack)
 template<typename selector_type>
 void basic_server<selector_type>::send(const packet& pack, const user& to)
 {
-	// Get selector from base class
-	selector_type& selector = basic_object<selector_type>::get_selector();
-	const tcp_client_socket& user_socket = to.get_connection().get_socket();
-
-	// Set OUTGOING flag if it isn't already
-	if(!selector.check(user_socket, socket::OUTGOING) )
-		selector.add(user_socket, socket::OUTGOING);
-
 	// Enqueue packet
 	to.send(pack);
 }
 
+template<typename selector_type>
+void basic_server<selector_type>::request_encryption(const user& to)
+{
+	to.request_encryption();
+}
+
 template<typename selector_type>
 const tcp_server_socket& basic_server<selector_type>::get_socket() const
 {
@@ -364,16 +386,6 @@ void basic_server<selector_type>::remove_client(const user* user)
 		on_part(*user);
 	on_disconnect(*user);
 
-	// Get selector from base class
-	selector_type& selector = basic_object<selector_type>::get_selector();
-	const tcp_client_socket& user_socket = 
-		user->get_connection().get_socket();
-
-	// Remove dead client from selector
-	selector.remove(user_socket, socket::INCOMING | socket::IOERROR);
-	if(selector.check(user_socket, socket::OUTGOING) )
-		selector.remove(user_socket, socket::OUTGOING);
-
 	// Store ID of client to remove
 	unsigned int user_id = user->get_id();
 	// Remove user to prevent server from sending the packet to the
@@ -390,72 +402,61 @@ void basic_server<selector_type>::remove_client(const user* user)
 }
 
 template<typename selector_type>
-void basic_server<selector_type>::on_accept_event(socket::condition io)
+void basic_server<selector_type>::on_accept_event(tcp_server_socket& sock,
+                                                  io_condition io)
 {
-	// Accept new client connection
-	user* client;
-	if(use_ipv6)
-	{
-		ipv6_address addr;
-		tcp_client_socket new_sock = serv_sock->accept(addr);
-		client = new user(
-			++ id_counter,
-			new connection(new_sock, addr)
-		);
-	}
-	else
-	{
-		ipv4_address addr;
-		tcp_client_socket new_sock = serv_sock->accept(addr);
-		client = new user(
-			++ id_counter,
-			new connection(new_sock, addr)
-		);
-	}
-
 	// Get selector from base class
 	selector_type& selector = basic_object<selector_type>::get_selector();
+	connection_type* conn = new connection_type(selector);
+	std::auto_ptr<user> client(new user(++ id_counter, conn) );
 
-	// Add new client into user list
-	basic_object<selector_type>::user_add(client);
-
-	// Add to selector to receive data from this client
-	connection& conn = client->get_connection();
-	selector.add(conn.get_socket(), socket::INCOMING | socket::IOERROR);
-
-	// Connect signal handlers
-	conn.send_event().connect(
-		sigc::bind(
-			sigc::mem_fun(*this, &basic_server::on_send_event),
-			sigc::ref(*client)
-		)
-	);
-
-	conn.recv_event().connect(
+	conn->recv_event().connect(
 		sigc::bind(
 			sigc::mem_fun(*this, &basic_server::on_recv_event),
 			sigc::ref(*client)
 		)
 	);
 
-	conn.close_event().connect(
+	conn->close_event().connect(
 		sigc::bind(
 			sigc::mem_fun(*this, &basic_server::on_close_event),
 			sigc::ref(*client)
 		)
 	);
 
+	conn->encrypted_event().connect(
+		sigc::bind(
+			sigc::mem_fun(*this, &basic_server::on_encrypted_event),
+			sigc::ref(*client)
+		)
+	);
+
+	conn->set_dh_params(params);
+
+	if(&sock == serv_sock.get())
+	{
+		ipv4_address addr;
+		std::auto_ptr<tcp_client_socket> new_sock(sock.accept(addr));
+		conn->assign(new_sock, addr);
+	}
+	else if(&sock == serv6_sock.get())
+	{
+		ipv6_address addr;
+		std::auto_ptr<tcp_client_socket> new_sock(sock.accept(addr));
+		conn->assign(new_sock, addr);
+	}
+	else
+	{
+		throw std::logic_error(
+			"net6::basic_server::on_accept_event:\n"
+			"Accept is nor from ipv4 neither from ipv6 socket"
+		);
+	}
+
+	basic_object<selector_type>::user_add(client.get() );
+
 	// Emit connection signal for new client
-	on_connect(*client);
-}
-
-template<typename selector_type>
-void basic_server<selector_type>::on_send_event(user& to)
-{
-	selector_type& selector = basic_object<selector_type>::get_selector();
-
-	// Remove OUTGOING from selector as no more data has to be sent
-	selector.remove(to.get_connection().get_socket(), socket::OUTGOING);
+	on_connect(*client.release() );
 }
 
 template<typename selector_type>
@@ -474,6 +475,20 @@ void basic_server<selector_type>::on_close_event(user& user)
 	remove_client(&user);
 }
 
+template<typename selector_type>
+void basic_server<selector_type>::on_encrypted_event(user& user)
+{
+	user.set_encrypted();
+
+	if(user.is_logged_in() )
+	{
+		// Tell about encrypted connection
+		net6::packet encr_pack("net6_encryption_info");
+		encr_pack << user.get_id();
+		send(encr_pack);
+	}
+}
+
 template<typename selector_type>
 void basic_server<selector_type>::on_connect(const user& user)
 {
@@ -521,11 +536,20 @@ void basic_server<selector_type>::on_login_extend(const user& user, packet& pack
 template<typename selector_type>
 void basic_server<selector_type>::on_data(const user& user, const packet& pack)
 {
-	signal_data.emit(user, pack);
+	try
+	{
+		signal_data.emit(user, pack);
+	}
+	catch(bad_packet& e)
+	{
+		// TODO: Print reason to stderr?
+		remove_client(&user);
+	}
 }
 
 template<typename selector_type>
-void basic_server<selector_type>::net_client_login(user& user, const packet& pack)
+void basic_server<selector_type>::
+	net_client_login(user& user, const packet& pack)
 {
 	// Is already logged in
 	if(user.is_logged_in() ) return;
@@ -567,7 +591,7 @@ void basic_server<selector_type>::net_client_login(user& user, const packet& pac
 
 		// Synchronise with other clients
 		packet self_pack("net6_client_join");
-		self_pack << user.get_id() << name;
+		self_pack << user.get_id() << name << user.is_encrypted();
 		on_login_extend(user, self_pack);
 		send(self_pack, user);
 
@@ -581,7 +605,8 @@ void basic_server<selector_type>::net_client_login(user& user, const packet& pac
 
 			packet join_pack("net6_client_join");
 			join_pack << iter->second->get_id()
-			          << iter->second->get_name();
+			          << iter->second->get_name()
+			          << iter->second->is_encrypted();
 			on_login_extend(*iter->second, join_pack);
 
 			send(join_pack, user);
@@ -596,33 +621,74 @@ void basic_server<selector_type>::net_client_login(user& user, const packet& pac
 template<typename selector_type>
 void basic_server<selector_type>::shutdown_impl()
 {
+	for(typename basic_object<selector_type>::user_const_iterator iter =
+		basic_object<selector_type>::users.begin();
+	    iter != basic_object<selector_type>::users.end();
+	    ++ iter)
+	{
+		delete iter->second;
+	}
+
+	basic_object<selector_type>::users.clear();
 	selector_type& selector = basic_object<selector_type>::get_selector();
 
-	selector.remove(*serv_sock, socket::INCOMING);
-	serv_sock.reset(NULL);
+	if(serv_sock.get() != NULL)
+	{
+		selector.set(*serv_sock, IO_NONE);
+		serv_sock.reset(NULL);
+	}
+
+	if(serv6_sock.get() != NULL)
+	{
+		selector.set(*serv6_sock, IO_NONE);
+		serv6_sock.reset(NULL);
+	}
 }
 
 template<typename selector_type>
 void basic_server<selector_type>::reopen_impl(unsigned int port)
 {
-	// Open socket on local port
-	if(use_ipv6)
-	{
-		ipv6_address bind_addr(port);
-		serv_sock.reset(new tcp_server_socket(bind_addr) );
-	}
-	else
+	selector_type& selector = basic_object<selector_type>::get_selector();
+
+	// Open IPv4 socket on local port
+	if(!use_ipv6)
 	{
 		ipv4_address bind_addr(port);
 		serv_sock.reset(new tcp_server_socket(bind_addr) );
+
+		selector.set(*serv_sock,
+			selector.get(*serv_sock) | IO_INCOMING
+		);
+
+		serv_sock->io_event().connect(
+			sigc::bind<0>(
+				sigc::mem_fun(
+					*this,
+					&basic_server::on_accept_event
+				),
+				sigc::ref(*serv_sock)
+			)
+		);
 	}
+	else
+	{
+		ipv6_address bind_addr(port);
+		serv6_sock.reset(new tcp_server_socket(bind_addr) );
 
-	selector_type& selector = basic_object<selector_type>::get_selector();
+		selector.set(*serv6_sock,
+			selector.get(*serv6_sock) | IO_INCOMING
+		);
 
-	// Add incoming flag to selector to accept client connections
-	selector.add(*serv_sock, socket::INCOMING);
-	serv_sock->io_event().connect(
-		sigc::mem_fun(*this, &basic_server::on_accept_event) );
+		serv6_sock->io_event().connect(
+			sigc::bind<0>(
+				sigc::mem_fun(
+					*this,
+					&basic_server::on_accept_event
+				),
+				sigc::ref(*serv6_sock)
+			)
+		);
+	}
 }
 
 } // namespace net6
