@@ -24,6 +24,7 @@
 #include <sstream>
 
 #include "common.hpp"
+#include "config.hpp"
 #include "error.hpp"
 #include "address.hpp"
 
@@ -68,20 +69,10 @@ namespace
 	}
 #endif
 
-#ifdef WIN32
-	// There is no inet_ntop on WIN32
-	inline const char *inet_ntop(int af, const void *__restrict src,
-	                             char *__restrict dest, socklen_t size)
+#ifdef _WIN32
+	inline const char* inet_ntop4(int af, const void *__restrict src,
+	                              char *__restrict dest, socklen_t size)
 	{
-		// IPV6 not supported (yet?)
-		if(AF_INET != af)
-		{
-			throw std::runtime_error(
-				"inet_ntop is only implemented for AF_INET "
-				"address family on win32"
-			);
-		}
-
 		// Format address
 		char* s = inet_ntoa(*reinterpret_cast<const in_addr*>(src));
 		if(s == NULL) return NULL;
@@ -90,6 +81,25 @@ namespace
 		socklen_t len = static_cast<socklen_t>(strlen(s) );
 		return strncpy(dest, s, size < (len + 1) ? size : (len + 1) );
 	}
+
+	// There is no inet_ntop on WIN32
+	inline const char *inet_ntop(int af, const void *__restrict src,
+	                             char *__restrict dest, socklen_t size)
+	{
+		switch(af)
+		{
+		case AF_INET:
+			return inet_ntop4(af, src, dest, size);
+			break;
+		default:
+			throw std::logic_error(
+				"inet_ntop is only implemented for "
+				"AF_INET address family on win32"
+			);
+
+			break;
+		}
+	}
 #endif
 }
 
@@ -143,7 +153,11 @@ net6::ipv4_address::create_from_hostname(const std::string& hostname,
 #ifdef WIN32
 	hostent* info = resolve_generic(hostname.c_str(), AF_INET, 0);
 #else
-	addrinfo* info = resolve_generic(hostname.c_str(), PF_INET, 0);
+#ifdef HAVE_AI_ADDRCONFIG
+	addrinfo* info = resolve_generic(hostname.c_str(), AF_INET, AI_ADDRCONFIG);
+#else
+	addrinfo* info = resolve_generic(hostname.c_str(), AF_INET, 0);
+#endif
 	sockaddr_in* ai_addr = reinterpret_cast<sockaddr_in*>(info->ai_addr);
 #endif
 
@@ -196,7 +210,11 @@ net6::ipv4_address::list(const std::string& hostname, unsigned int port)
 #ifdef WIN32
 	hostent* info = resolve_generic(hostname.c_str(), AF_INET, 0);
 #else
-	addrinfo* info = resolve_generic(hostname.c_str(), PF_INET, 0);
+#ifdef HAVE_AI_ADDRCONFIG
+	addrinfo* info = resolve_generic(hostname.c_str(), AF_INET, AI_ADDRCONFIG);
+#else
+	addrinfo* info = resolve_generic(hostname.c_str(), AF_INET, 0);
+#endif
 #endif
 
 #ifdef WIN32
@@ -325,7 +343,11 @@ net6::ipv6_address::create_from_hostname(const std::string& hostname,
 #ifdef WIN32
 	hostent* info = resolve_generic(hostname.c_str(), AF_INET6, 0);
 #else
-	addrinfo* info = resolve_generic(hostname.c_str(), PF_INET6, 0);
+#ifdef HAVE_AI_ADDRCONFIG
+	addrinfo* info = resolve_generic(hostname.c_str(), AF_INET6, AI_ADDRCONFIG);
+#else
+	addrinfo* info = resolve_generic(hostname.c_str(), AF_INET6, 0);
+#endif
 	sockaddr_in6* ai_addr = reinterpret_cast<sockaddr_in6*>(info->ai_addr);
 #endif
 
@@ -409,7 +431,11 @@ net6::ipv6_address::list(const std::string& hostname, unsigned int port,
 #ifdef WIN32
 	hostent* info = resolve_generic(hostname.c_str(), AF_INET6, 0);
 #else
-	addrinfo* info = resolve_generic(hostname.c_str(), PF_INET6, 0);
+#ifdef HAVE_AI_ADDRCONFIG
+	addrinfo* info = resolve_generic(hostname.c_str(), AF_INET6, AI_ADDRCONFIG);
+#else
+	addrinfo* info = resolve_generic(hostname.c_str(), AF_INET6, 0);
+#endif
 #endif
 
 #ifdef WIN32
@@ -491,11 +517,30 @@ net6::address* net6::ipv6_address::clone() const
 
 std::string net6::ipv6_address::get_name() const
 {
+#ifdef _WIN32
+	// No call to inet_ntop on WIN32 since inet_ntop is not defined
+	// on Win32. Use WSAAddressToString instead.
+	char destbuf[INET6_ADDRSTRLEN + 6];
+	DWORD len = INET6_ADDRSTRLEN + 6;
+
+	WSAAddressToStringA(
+		addr,
+		sizeof(sockaddr_in6),
+		NULL,
+		destbuf,
+		&len
+	);
+
+	char* sep = strrchr(destbuf, ':');
+	if(sep != NULL) *sep = '\0';
+
+	return destbuf;
+#else
 	const size_t len = INET6_ADDRSTRLEN;
 	char buf[len];
-
 	inet_ntop(AF_INET6, &((sockaddr_in6*)addr)->sin6_addr, buf, len);
 	return buf;
+#endif
 }
 
 socklen_t net6::ipv6_address::get_size() const
